﻿using System;

namespace ProcessHang {

    [Android.App.Activity(Label = "ProcessHang", MainLauncher = true)]
    public class MainActivity : Android.App.Activity {

        protected override void OnCreate(Android.OS.Bundle bundle) {
            base.OnCreate(bundle);

            SetContentView(Resource.Layout.Main);

            var runTestButton = FindViewById<Android.Widget.Button>(Resource.Id.runTestButton);            
            runTestButton.Click += delegate {
                Console.WriteLine("TEST START ------------------");

                for(int i = 0; i < 10; ++i) {
                    UseMonoProcess();
                }

                RunJavaProcess();

                for(int i = 0; i < 10; ++i) {
                    UseMonoProcess();
                }

                Console.WriteLine("TEST END ------------------");
            };
        }

        // Invokes the 'id' command using the Mono Process class

        // Example adapted from ping.cs: https://github.com/mono/mono/blob/master/mcs/class/System/System.Net.NetworkInformation/Ping.cs
        static void UseMonoProcess() {
            using (var monoProcess = new System.Diagnostics.Process()) {

                monoProcess.StartInfo.FileName = "id";

                monoProcess.StartInfo.CreateNoWindow = true;
                monoProcess.StartInfo.UseShellExecute = false;

                monoProcess.StartInfo.RedirectStandardOutput = true;
                monoProcess.StartInfo.RedirectStandardError = true;

                monoProcess.Start();

                var stdout = monoProcess.StandardOutput.ReadToEnd();
                var stderr = monoProcess.StandardError.ReadToEnd();

                // Wait for 5 seconds before aborting process run
                if (!monoProcess.WaitForExit(5000)) {
                    Console.WriteLine("FAILURE MONO: TIMEOUT!");
                } else if (monoProcess.ExitCode == 1) {
                    Console.WriteLine("ERROR: " + stderr);
                } else {
                    Console.WriteLine("SUCCESS MONO: " + stdout);
                }
            }
        }

        // Invokes the 'id' command using the Java Process class

        static void RunJavaProcess() {
            using (var procBuilder = new Java.Lang.ProcessBuilder().Command("id").RedirectErrorStream(true))
            using (var proc = procBuilder.Start())
            using (var inputStream = new System.IO.StreamReader(proc.InputStream)) {
                var resultString = inputStream.ReadToEnd();
                Console.WriteLine("SUCCESS JAVA: " + resultString);
                // This line hangs in a similar way to the Mono process hang, but there are no timeout options for the native process
//                var exitCode = proc.WaitFor();
//                Console.WriteLine("EXIT JAVA: " + exitCode);
            }
        }

    }
}


